
' Object for managing the Bookmarks
Public Class BookmarkManager
    Private m_root As FolderEntry

    ' Default Constructor to initialize the BookmarkManager
    Public Sub New()
        m_root = New FolderEntry()                                  ' create the root folder
        m_root.Name = "Root"                                        ' name it
        m_root.Description = "The root of all the bookmark folders" ' describe it
    End Sub

    ' Property to access the root folder
    Public ReadOnly Property RootFolder() As FolderEntry
        Get
            Return m_root
        End Get
    End Property

    ' Procedure to load bookmarks from a file
    Public Sub getEntries(ByVal path As String)
        Dim doc As New Xml.XmlDocument

        Try
            doc.Load(path)                                              ' load the xml data file

            loadFolder(doc.DocumentElement, m_root)                     ' begin recursive folder loading
        Catch Ex As IO.FileNotFoundException
            Dim newFolder As New FolderEntry
            newFolder.Name = "BookIt"

            Dim newBookmark As New BookmarkEntry
            newBookmark.Name = "Software Engineering"
            newBookmark.URL = "http://www.kglan.com/se1/index.php"
            newFolder.addBookmarkEntry(newBookmark)

            m_root.addFolderEntry(newFolder)
        End Try
    End Sub

    ' Procedure to save the bookmarks file
    Public Sub saveEntries(ByVal path As String)
        Dim xw As New Xml.XmlTextWriter(path, System.Text.UTF8Encoding.UTF8)
        Dim folder As FolderEntry

        xw.Formatting = Xml.Formatting.Indented
        xw.WriteStartDocument()

        xw.WriteStartElement("bookmarks")
        xw.WriteAttributeString("version", "1.0")

        For Each folder In m_root.SubFolders
            xw.WriteStartElement("group")
            xw.WriteAttributeString("title", folder.Name)

            saveFolder(xw, folder)

            xw.WriteEndElement()
        Next

        xw.WriteEndElement()
        xw.Close()
    End Sub

    ' Recursive function to load an XML element to it's corresponding folder
    Private Sub loadFolder(ByVal Element As Xml.XmlElement, ByVal Folder As FolderEntry)
        Dim E As Xml.XmlElement

        For Each E In Element.ChildNodes                            ' loop through the child xml nodes
            If E.Name = "group" Then                                ' if the xml node is a folder
                Dim newFolder As FolderEntry                        ' create a new FolderEntry object
                newFolder = New FolderEntry
                newFolder.Name = E.GetAttribute("title")            ' name it

                Folder.addFolderEntry(newFolder)                    ' add the entry to the parent folder

                loadFolder(E, newFolder)                            ' because this node represents a folder, call loadFolder to load it

            ElseIf E.Name = "bookmark" Then                         ' if the xml node is a bookmark
                Dim newEntry As BookmarkEntry                       ' create a new BookmarkEntry
                newEntry = New BookmarkEntry
                newEntry.Name = E.GetAttribute("title")             ' name it
                newEntry.URL = E.GetAttribute("href")               ' assign the url to it
                newEntry.Starred = E.HasAttribute("fav")            ' if the "fav" attribute is present then it is a favorite

                If E.IsEmpty = False Then
                    newEntry.Description = E.InnerText.Trim         ' describe it if a description is available
                End If

                Folder.addBookmarkEntry(newEntry)                   ' add the BookmarkEntry to the FolderEntry
            End If
        Next
    End Sub

    Private Sub saveFolder(ByVal xw As Xml.XmlTextWriter, ByVal MainFolder As FolderEntry)
        Dim folder As FolderEntry
        Dim bookmark As BookmarkEntry

        For Each folder In MainFolder.SubFolders
            xw.WriteStartElement("group")
            xw.WriteAttributeString("title", folder.Name)

            saveFolder(xw, folder)

            xw.WriteEndElement()
        Next
        For Each bookmark In MainFolder.BookmarkEntries
            xw.WriteStartElement("bookmark")
            xw.WriteAttributeString("href", bookmark.URL)
            xw.WriteAttributeString("title", bookmark.Name)
            If bookmark.Starred Then
                xw.WriteAttributeString("fav", "fav")
            End If
            If bookmark.Description <> "" Then
                xw.WriteString(bookmark.Description)
            End If
            xw.WriteEndElement()
        Next
    End Sub

    Public Sub moveBookmark(ByVal bookmark As BookmarkEntry, ByVal newParent As FolderEntry)
        bookmark.Parent.deleteBookmarkEntry(bookmark)
        newParent.addBookmarkEntry(bookmark)
    End Sub

    Public Sub moveFolder(ByVal folder As FolderEntry, ByVal newParent As FolderEntry)
        folder.Parent.deleteFolderEntry(folder)
        newParent.addFolderEntry(folder)
    End Sub

    Public Sub importFromBrowser()

    End Sub

    Public Sub exportToBrowser()

    End Sub
End Class

' Object for holding Folder data
Public Class FolderEntry
    ' Local variables and property data
    Private m_name As String                                        ' Name of the FolderEntry
    Private m_description As String                                 ' Description of the Entry
    Private m_bookmarkEntries As New List(Of BookmarkEntry)         ' The list of BookmarkEntry's
    Private m_subFolders As New List(Of FolderEntry)                ' The list of Subfolders
    Private m_parent As FolderEntry                                 ' The Parent to the current Folder

#Region " Properties "
    ' Name of the FolderEntry
    Public Property Name() As String
        Get
            Return m_name
        End Get
        Set(ByVal value As String)
            m_name = value
        End Set
    End Property

    ' FolderEntry description
    Public Property Description() As String
        Get
            Return m_description
        End Get
        Set(ByVal value As String)
            m_description = value
        End Set
    End Property

    ' Subfolder list in the FolderEntry
    Public ReadOnly Property SubFolders() As List(Of FolderEntry)
        Get
            Return m_subFolders
        End Get
    End Property

    ' BookmarkEntries list
    Public ReadOnly Property BookmarkEntries() As List(Of BookmarkEntry)
        Get
            Return m_bookmarkEntries
        End Get
    End Property

    ' The Parent folder of this folder
    Public Property Parent() As FolderEntry
        Get
            Return m_parent
        End Get
        Set(ByVal value As FolderEntry)
            m_parent = value
        End Set
    End Property

    Public ReadOnly Property Path() As String
        Get
            If m_parent Is Nothing Then
                Return ""
            Else
                Return m_parent.Path & m_name & "/"
            End If
        End Get
    End Property
#End Region

    ' Adds a BookmarkEntry to the FolderEntry
    Public Sub addBookmarkEntry(ByVal newEntry As BookmarkEntry)
        newEntry.Parent = Me
        m_bookmarkEntries.Add(newEntry)
    End Sub

    Public Sub deleteBookmarkEntry(ByVal bookmark As BookmarkEntry)
        m_bookmarkEntries.Remove(bookmark)
        bookmark.Parent = Nothing
    End Sub

    ' Adds a FolderEntry as a sub folder of the FolderEntry
    Public Sub addFolderEntry(ByVal newFolder As FolderEntry)
        newFolder.Parent = Me
        m_subFolders.Add(newFolder)
    End Sub

    Public Sub deleteFolderEntry(ByVal folder As FolderEntry)
        m_subFolders.Remove(folder)
        folder.Parent = Nothing
    End Sub

    ' Clears the treeview and launches the shared displayHierarchy for all the subfolders
    '   of the folder represented by the instance of the FolderEntry on which this function
    '   is called
    Public Sub displayHierarchy(ByVal tv As TreeView, ByVal DefaultSelection As FolderEntry)
        Dim subFolder As FolderEntry

        tv.Nodes.Clear()                                            ' Clear the entier treeview control

        For Each subFolder In Me.SubFolders                         ' loop through this folder's subfolders
            displayHierarchy(tv.Nodes, subFolder, DefaultSelection)                   ' initiate the recursive function to load the subfolders
        Next
    End Sub

    ' Recursive function to build a treeview of the FolderEntry's
    ' tvNodes is the Nodes collection of the TreeNode that represents the folder argument
    Public Shared Sub displayHierarchy(ByVal tvNodes As TreeNodeCollection, ByVal folder As FolderEntry, ByVal DefaultSelection As FolderEntry)
        Dim newNode As TreeNode                                     ' Variable to hold the new TreeNode
        Dim subFolder As FolderEntry                                ' Variable used to loop through subfolders
        newNode = New TreeNode(folder.Name)                         ' create the folder's node
        newNode.Tag = folder                                        ' tag the TreeNode object with the folder itself

        tvNodes.Add(newNode)                                        ' add the newNode to the Nodes collection

        If Object.ReferenceEquals(folder, DefaultSelection) Then
            newNode.TreeView.SelectedNode = newNode
        End If

        For Each subFolder In folder.SubFolders                     ' loop through all the subfolders
            displayHierarchy(newNode.Nodes, subFolder, DefaultSelection)              ' initiate the recursive function to load the subfolders
        Next
    End Sub
End Class

' Object for holding Bookmark data
Public Class BookmarkEntry
    ' Local variables for property data
    Private m_description As String                                 ' Description
    Private m_name As String = ""                                   ' Name
    Private m_url As String                                         ' URL
    Private m_starred As Boolean                                    ' Starred
    Private m_parent As FolderEntry                                 ' The Containing folder

    ' The description of the BookmarkEntry
    Public Property Description() As String
        Get
            Return m_description
        End Get
        Set(ByVal value As String)
            m_description = value
        End Set
    End Property

    ' The name of the BookmarkEntry
    Public Property Name() As String
        Get
            Return m_name
        End Get
        Set(ByVal value As String)
            m_name = value
        End Set
    End Property

    ' The URL the BookmarkEntry is addressing
    Public Property URL() As String
        Get
            Return m_url
        End Get
        Set(ByVal value As String)
            m_url = value
        End Set
    End Property

    ' Defines whether the BookmarkEntry is a favorite
    Public Property Starred() As Boolean
        Get
            Return m_starred
        End Get
        Set(ByVal value As Boolean)
            m_starred = value
        End Set
    End Property

    Public Property Parent() As FolderEntry
        Get
            Return m_parent
        End Get
        Set(ByVal value As FolderEntry)
            m_parent = value
        End Set
    End Property
End Class